@@ -236,7 +236,7 @@ buf_Sync(int quitOnShutdown)
             afs_uint32 dirty;
             cm_volume_t * volp;
 
-            volp = cm_GetVolumeByFID(&bp->fid);
+	    volp = cm_FindVolumeByFID(&bp->fid, bp->userp, &req);
             switch (cm_GetVolumeStatus(volp, bp->fid.volume)) {
             case vl_online:
             case vl_unknown:
@@ -277,7 +277,7 @@ buf_Sync(int quitOnShutdown)
                 char volstr[VL_MAXNAMELEN+12]="";
                 char *ext = "";
 
-                volp = cm_GetVolumeByFID(&bp->fid);
+		volp = cm_GetVolumeByFID(&bp->fid);
                 if (volp) {
                     cellp = volp->cellp;
                     if (bp->fid.volume == volp->vol[RWVOL].ID)
@@ -348,6 +348,14 @@ buf_ValidateBuffers(void)
     }
 
     for (bp = cm_data.buf_freeListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -11;
+	}
+
         if (bp->magic != CM_BUF_MAGIC) {
             afsi_log("cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC");
             fprintf(stderr, "cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC\n");
@@ -364,6 +372,14 @@ buf_ValidateBuffers(void)
     }
 
     for (bp = cm_data.buf_freeListp; bp; bp=(cm_buf_t *) osi_QNext(&bp->q)) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -12;
+	}
+
         if (bp->magic != CM_BUF_MAGIC) {
             afsi_log("cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC");
             fprintf(stderr, "cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC\n");
@@ -380,6 +396,14 @@ buf_ValidateBuffers(void)
     }
 
     for ( bp = cm_data.buf_redirListp; bp; bp = (cm_buf_t *) osi_QNext(&bp->q)) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -13;
+	}
+
         if (!(bp->qFlags & CM_BUF_QREDIR)) {
             afsi_log("CM_BUF_QREDIR not set on cm_buf_t in buf_redirListp");
             fprintf(stderr, "CM_BUF_QREDIR not set on cm_buf_t in buf_redirListp");
@@ -394,11 +418,27 @@ buf_ValidateBuffers(void)
     }
 
     for (bp = cm_data.buf_allp; bp; bp=bp->allp) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -14;
+	}
+
         if (bp->magic != CM_BUF_MAGIC) {
             afsi_log("cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC");
             fprintf(stderr, "cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC\n");
             return -3;
         }
+
+	if ( bp->datap < cm_data.bufDataBaseAddress ||
+	     bp->datap >= cm_data.bufEndOfData) {
+	    afsi_log("cm_ValidateBuffers failure: out of range data pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range data pointers\n");
+	    return -15;
+	}
+
         counta++;
         bpa = bp;
 
@@ -819,13 +859,17 @@ afs_uint32 buf_CleanLocked(cm_scache_t *scp, cm_buf_t *bp, cm_req_t *reqp,
     {
         release_scp = 1;
 
-        lock_ObtainWrite(&scp->rw);
-        code = cm_SyncOp(scp, NULL, bp->userp ? bp->userp : cm_rootUserp, reqp, 0,
-                         CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
-        if (code == 0) {
-            cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+	if (scp->flags & CM_SCACHEFLAG_DELETED) {
+	    code = CM_ERROR_NOSUCHFILE;
+	} else {
+	    lock_ObtainWrite(&scp->rw);
+	    code = cm_SyncOp(scp, NULL, bp->userp ? bp->userp : cm_rootUserp, reqp, 0,
+			    CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+	    if (code == 0) {
+		cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+	    }
+	    lock_ReleaseWrite(&scp->rw);
         }
-        lock_ReleaseWrite(&scp->rw);
     }
 
     if (scp && (scp->flags & CM_SCACHEFLAG_DELETED)) {
@@ -2559,37 +2603,6 @@ long buf_ClearRDRFlag(cm_scache_t *scp, char *reason)
     return 0;
 }
 
-#if 0
-long buf_CleanDirtyBuffers(cm_scache_t *scp)
-{
-    cm_buf_t *bp;
-    afs_uint32 bcount = 0;
-    cm_fid_t * fidp = &scp->fid;
-
-    for (bp = cm_data.buf_allp; bp; bp=bp->allp, bcount++) {
-	if (!cm_FidCmp(fidp, &bp->fid) && (bp->flags & CM_BUF_DIRTY)) {
-            buf_Hold(bp);
-	    lock_ObtainMutex(&bp->mx);
-	    _InterlockedAnd(&bp->cmFlags, ~CM_BUF_CMSTORING);
-	    _InterlockedAnd(&bp->flags, ~CM_BUF_DIRTY);
-            bp->dirty_length = 0;
-	    _InterlockedOr(&bp->flags, CM_BUF_ERROR);
-	    bp->error = VNOVNODE;
-	    bp->dataVersion = CM_BUF_VERSION_BAD; /* bad */
-	    bp->dirtyCounter++;
-	    if (bp->flags & CM_BUF_WAITING) {
-		osi_Log2(buf_logp, "BUF CleanDirtyBuffers Waking [scp 0x%x] bp 0x%x", scp, bp);
-		osi_Wakeup((long) &bp);
-	    }
-	    lock_ReleaseMutex(&bp->mx);
-	    buf_Release(bp);
-            buf_DecrementUsedCount();
-	}
-    }
-    return 0;
-}
-#endif
-
 /*
  * The following routines will not be used on a
  * regular basis but are very useful in a variety
