@@ -263,19 +263,16 @@ cm_PingServer(cm_server_t *tsp)
     cm_req_t req;
 
     lock_ObtainMutex(&tsp->mx);
-    if (tsp->flags & CM_SERVERFLAG_PINGING) {
+    if (InterlockedIncrement(&tsp->pingCount) > 1) {
 	tsp->waitCount++;
 	osi_SleepM((LONG_PTR)tsp, &tsp->mx);
 	lock_ObtainMutex(&tsp->mx);
-	tsp->waitCount--;
-	if (tsp->waitCount == 0)
-	    _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
-	else
+	InterlockedDecrement(&tsp->pingCount);
+	if (--tsp->waitCount > 0)
 	    osi_Wakeup((LONG_PTR)tsp);
 	lock_ReleaseMutex(&tsp->mx);
 	return;
     }
-    _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PINGING);
     wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
     afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
     lock_ReleaseMutex(&tsp->mx);
@@ -313,14 +310,13 @@ cm_PingServer(cm_server_t *tsp)
     }	/* got an unauthenticated connection to this server */
 
     lock_ObtainMutex(&tsp->mx);
-    if (code >= 0 || code == RXGEN_OPCODE || code == RX_CALL_BUSY) {
+    if (code >= 0 || code == RXGEN_OPCODE) {
 	/* mark server as up */
 	_InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
         tsp->downTime = 0;
 
 	/* we currently handle 32-bits of capabilities */
-	if (code != RXGEN_OPCODE && code != RX_CALL_BUSY &&
-            caps.Capabilities_len > 0) {
+	if (code != RXGEN_OPCODE && caps.Capabilities_len > 0) {
 	    tsp->capabilities = caps.Capabilities_val[0];
 	    xdr_free((xdrproc_t) xdr_Capabilities, &caps);
 	    caps.Capabilities_len = 0;
@@ -367,9 +363,8 @@ cm_PingServer(cm_server_t *tsp)
 		  tsp->capabilities);
     }
 
-    if (tsp->waitCount == 0)
-	_InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
-    else
+    InterlockedDecrement(&tsp->pingCount);
+    if (tsp->waitCount > 0)
 	osi_Wakeup((LONG_PTR)tsp);
     lock_ReleaseMutex(&tsp->mx);
 }
@@ -520,7 +515,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             isDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if ((tsp->flags & CM_SERVERFLAG_PINGING) ||
+	    if (tsp->pingCount > 0 ||
                 !((isDown && (flags & CM_FLAG_CHECKDOWNSERVERS)) ||
                    (!isDown && (flags & CM_FLAG_CHECKUPSERVERS)))) {
                 lock_ReleaseMutex(&tsp->mx);
@@ -529,20 +524,20 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                 continue;
             }
 
-            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PINGING);
+	    InterlockedIncrement(&tsp->pingCount);
             lock_ReleaseMutex(&tsp->mx);
 
-            serversp[nconns] = tsp;
 	    if (cm_noIPAddr > 0)
 		code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &conns[nconns]);
 	    else
 		code = RX_CALL_DEAD;
             if (code) {
-		if (code == RX_CALL_DEAD) {
-		    lock_ObtainMutex(&tsp->mx);
+		lock_ObtainMutex(&tsp->mx);
+		if (code == RX_CALL_DEAD)
 		    cm_MarkServerDown(tsp, code, isDown);
-		    lock_ReleaseMutex(&tsp->mx);
-		}
+		InterlockedDecrement(&tsp->pingCount);
+		lock_ReleaseMutex(&tsp->mx);
+
 		lock_ObtainRead(&cm_serverLock);
 		cm_PutServerNoLock(tsp);
                 continue;
@@ -551,7 +546,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             rxconns[nconns] = cm_GetRxConn(conns[nconns]);
             if (conntimer[nconns] = (isDown ? 1 : 0))
                 rx_SetConnHardDeadTime(rxconns[nconns], 10);
-
+	    serversp[nconns] = tsp;
             nconns++;
         }
         lock_ReleaseRead(&cm_serverLock);
@@ -579,15 +574,13 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if (results[i] >= 0 || results[i] == RXGEN_OPCODE ||
-                results[i] == RX_CALL_BUSY)  {
+            if (results[i] >= 0 || results[i] == RXGEN_OPCODE) {
                 /* mark server as up */
                 _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
                 tsp->downTime = 0;
 
                 /* we currently handle 32-bits of capabilities */
-                if (results[i] != RXGEN_OPCODE && results[i] != RX_CALL_BUSY &&
-                    caps[i].Capabilities_len > 0) {
+                if (results[i] != RXGEN_OPCODE && caps[i].Capabilities_len > 0) {
                     tsp->capabilities = caps[i].Capabilities_val[0];
                     xdr_free((xdrproc_t) xdr_Capabilities, &caps[i]);
                     caps[i].Capabilities_len = 0;
@@ -636,9 +629,8 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                           tsp->capabilities);
             }
 
-            if (tsp->waitCount == 0)
-                _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
-            else
+	    InterlockedDecrement(&tsp->pingCount);
+	    if (tsp->waitCount > 0)
                 osi_Wakeup((LONG_PTR)tsp);
 
             lock_ReleaseMutex(&tsp->mx);
@@ -665,7 +657,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             isDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if ((tsp->flags & CM_SERVERFLAG_PINGING) ||
+	    if (tsp->pingCount > 0 ||
                 !((isDown && (flags & CM_FLAG_CHECKDOWNSERVERS)) ||
                    (!isDown && (flags & CM_FLAG_CHECKUPSERVERS)))) {
                 lock_ReleaseMutex(&tsp->mx);
@@ -674,20 +666,20 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                 continue;
             }
 
-            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PINGING);
+	    InterlockedIncrement(&tsp->pingCount);
             lock_ReleaseMutex(&tsp->mx);
 
-            serversp[nconns] = tsp;
 	    if (cm_noIPAddr > 0)
 		code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &conns[nconns]);
 	    else
 		code = RX_CALL_DEAD;
             if (code) {
-		if (code == RX_CALL_DEAD) {
-		    lock_ObtainMutex(&tsp->mx);
+		lock_ObtainMutex(&tsp->mx);
+		if (code == RX_CALL_DEAD)
 		    cm_MarkServerDown(tsp, code, isDown);
-		    lock_ReleaseMutex(&tsp->mx);
-		}
+		InterlockedDecrement(&tsp->pingCount);
+		lock_ReleaseMutex(&tsp->mx);
+
 		lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
                 continue;
@@ -697,7 +689,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             conntimer[nconns] = (isDown ? 1 : 0);
             if (isDown)
                 rx_SetConnHardDeadTime(rxconns[nconns], 10);
-
+	    serversp[nconns] = tsp;
             nconns++;
         }
         lock_ReleaseRead(&cm_serverLock);
@@ -725,7 +717,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if (results[i] >= 0 || results[i] == RX_CALL_BUSY)  {
+            if (results[i] >= 0)  {
                 /* mark server as up */
                 _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
                 tsp->downTime = 0;
@@ -748,9 +740,8 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                           tsp->capabilities);
             }
 
-            if (tsp->waitCount == 0)
-                _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
-            else
+	    InterlockedDecrement(&tsp->pingCount);
+	    if (tsp->waitCount > 0)
                 osi_Wakeup((LONG_PTR)tsp);
 
             lock_ReleaseMutex(&tsp->mx);
@@ -774,6 +765,17 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
     DWORD dummyLen;
     DWORD multi = 1;
 
+    lock_ObtainRead(&cm_syscfgLock);
+    if (cm_LanAdapterChangeDetected) {
+	lock_ConvertRToW(&cm_syscfgLock);
+	if (cm_LanAdapterChangeDetected) {
+	    code = cm_UpdateIFInfo();
+	}
+	lock_ReleaseWrite(&cm_syscfgLock);
+    } else {
+	lock_ReleaseRead(&cm_syscfgLock);
+    }
+
     code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
                          0, KEY_QUERY_VALUE, &parmKey);
     if (code == ERROR_SUCCESS) {
@@ -870,7 +872,9 @@ afs_int32 cm_UpdateIFInfo(void)
     code = syscfg_GetIFInfo(&cm_noIPAddr,
 			     cm_IPAddr, cm_SubnetMask,
 			     cm_NetMtu, cm_NetFlags);
-    cm_LanAdapterChangeDetected = 0;
+
+    cm_LanAdapterChangeDetected = (code != 0);
+
     return code;
 }
 
@@ -1532,7 +1536,7 @@ void cm_AppendServerList(cm_serverRef_t *dest, cm_serverRef_t **src)
 {
     cm_serverRef_t *ref;
 
-    if (dest == NULL | src == NULL || *src == NULL)
+    if (dest == NULL || src == NULL || *src == NULL)
 	return;
 
     for (ref = dest; ref->next != NULL; ref = ref->next);
@@ -1638,10 +1642,12 @@ int cm_DumpServers(FILE *outputFile, char *cookie, int lock)
 
         sprintf(output,
                  "%s - tsp=0x%p cell=%s addr=%-15s port=%u uuid=%s type=%s caps=0x%x "
-                 "flags=0x%x waitCount=%u rank=%u downTime=\"%s\" refCount=%u\r\n",
+		 "flags=0x%x waitCount=%u pingCount=%d rank=%u downTime=\"%s\" "
+		 "refCount=%u\r\n",
                  cookie, tsp, tsp->cellp ? tsp->cellp->name : "", hoststr,
                  ntohs(tsp->addr.sin_port), uuidstr, type,
-                 tsp->capabilities, tsp->flags, tsp->waitCount, tsp->activeRank,
+		 tsp->capabilities, tsp->flags, tsp->waitCount, tsp->pingCount,
+		 tsp->activeRank,
                  (tsp->flags & CM_SERVERFLAG_DOWN) ?  "down" : "up",
                  tsp->refCount);
         WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
