@@ -33,6 +33,7 @@ typedef struct cm_server {
     afs_int32 capabilities;		/* by mx */
     struct cm_cell *cellp;		/* cell containing this server */
     afs_int32 refCount;		        /* Interlocked with cm_serverLock */
+    afs_int32 pingCount;		/* Interlocked by mx */
     osi_mutex_t mx;
     unsigned short ipRank;		/* network class rank */
     unsigned short adminRank;		/* set if admin sets a rank
@@ -63,7 +64,7 @@ typedef struct cm_serverRef {
 /* flags */
 #define CM_SERVERFLAG_DOWN	0x1	/* server is down */
 #define CM_SERVERFLAG_PREF_SET	0x2     /* server preference set by user */
-#define CM_SERVERFLAG_PINGING 	0x4 	/* a ping against this server in progress */
+
 #define CM_SERVERFLAG_NO64BIT   0x8     /* server has no support for
                                            64-bit operations. */
 #define CM_SERVERFLAG_NOINLINEBULK 0x10	/* server has no support for inline bulk */
@@ -123,6 +124,8 @@ extern void cm_RankUpServers();
 
 extern void cm_SetServerIPRank(cm_server_t * serverp);
 
+extern afs_int32 cm_UpdateIFInfo(void);
+
 extern void cm_InsertServerList(cm_serverRef_t** list,cm_serverRef_t* element);
 
 extern long cm_ChangeRankServer(cm_serverRef_t** list, cm_server_t* server);
@@ -137,6 +140,8 @@ extern afs_uint32 cm_ServerListSize(cm_serverRef_t* list);
 
 extern void cm_FreeServerList(cm_serverRef_t** list, afs_uint32 flags);
 
+extern void cm_AppendServerList(cm_serverRef_t *dest, cm_serverRef_t **src);
+
 extern void cm_ForceNewConnectionsAllServers(void);
 
 extern void cm_SetServerNo64Bit(cm_server_t * serverp, int no64bit);
