@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2008, 2009, 2010, 2011 Kernel Drivers, LLC.
- * Copyright (c) 2009, 2010, 2011 Your File System, Inc.
+ * Copyright (c) 2009, 2010, 2011, 2015 Your File System, Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -574,7 +574,8 @@ AFSLocateNameEntry( IN GUID *AuthGroup,
                     OUT LONG *OutVolumeReferenceReason,
                     OUT AFSDirectoryCB **OutParentDirectoryCB,
                     OUT AFSDirectoryCB **OutDirectoryCB,
-                    OUT PUNICODE_STRING ComponentName);
+                    OUT PUNICODE_STRING ComponentName,
+		    OUT PUNICODE_STRING TargetName);
 
 NTSTATUS
 AFSCreateDirEntry( IN GUID            *AuthGroup,
@@ -632,8 +633,16 @@ AFSBuildRootVolume( IN GUID *AuthGroup,
 NTSTATUS
 AFSProcessDFSLink( IN AFSDirectoryCB *DirEntry,
                    IN PFILE_OBJECT FileObject,
-                   IN UNICODE_STRING *RemainingPath,
-                   IN GUID *AuthGroup);
+                   IN PUNICODE_STRING RemainingPath,
+                   IN GUID *AuthGroup,
+		   OUT PUNICODE_STRING TargetName);
+
+NTSTATUS
+AFSGetFullFileName( IN AFSFcb *Fcb,
+		    IN AFSCcb *Ccb,
+		    OUT ULONG *FileNameLength,
+		    OUT WCHAR *FileName,
+		    IN OUT LONG *RemainingLength);
 
 //
 // AFSNetworkProviderSupport.cpp
@@ -795,24 +804,6 @@ AFSQueryStreamInfo( IN PIRP Irp,
                     IN OUT FILE_STREAM_INFORMATION *Buffer,
                     IN OUT PLONG Length);
 
-NTSTATUS
-AFSQueryAccess( IN PIRP Irp,
-                IN AFSFcb *Fcb,
-                IN OUT PFILE_ACCESS_INFORMATION Buffer,
-                IN OUT PLONG Length);
-
-NTSTATUS
-AFSQueryMode( IN PIRP Irp,
-              IN AFSFcb *Fcb,
-              IN OUT PFILE_MODE_INFORMATION Buffer,
-              IN OUT PLONG Length);
-
-NTSTATUS
-AFSQueryAlignment( IN PIRP Irp,
-                   IN AFSFcb *Fcb,
-                   IN OUT PFILE_ALIGNMENT_INFORMATION Buffer,
-                   IN OUT PLONG Length);
-
 NTSTATUS
 AFSQueryAttribTagInfo( IN PIRP Irp,
                        IN AFSDirectoryCB *DirectoryCB,
@@ -1379,8 +1370,13 @@ void
 AFSRemoveNameEntry( IN AFSObjectInfoCB *ParentObjectInfo,
                     IN AFSDirectoryCB *DirEntry);
 
-LARGE_INTEGER
-AFSGetAuthenticationId( void);
+NTSTATUS
+AFSGetAuthenticationId( OUT LARGE_INTEGER *pliAuthId);
+
+NTSTATUS
+AFSPerformGetAuthId( IN PEPROCESS peProcess,
+		     IN PETHREAD peThread,
+		     OUT LARGE_INTEGER *outAuthId);
 
 void
 AFSUnwindFileInfo( IN AFSFcb *Fcb,
@@ -1456,6 +1452,13 @@ AFSPerformObjectInvalidate( IN AFSObjectInfoCB *ObjectInfo,
 BOOLEAN
 AFSIgnoreReparsePointToFile( void);
 
+NTSTATUS
+AFSRetrieveTargetFileInfo( IN PUNICODE_STRING TargetName,
+			   OUT AFSFileInfoCB *FileInfo);
+
+BOOLEAN
+AFSIsShareName( IN UNICODE_STRING *FileName);
+
 //
 // AFSNameArray.cpp Prototypes
 //
