@@ -1,20 +1,19 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
 #include <afs/stds.h>
 
+#include <roken.h>
+
 #include <windows.h>
-#include <stdlib.h>
-#include <stddef.h>
-#include <string.h>
-#include <stdio.h>
 
 #include "afsreg.h"
 
@@ -42,7 +41,7 @@ int IsWow64(void)
         hModule = GetModuleHandle(TEXT("kernel32"));
         if (hModule) {
             fnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress(hModule, "IsWow64Process");
-  
+
             if (NULL != fnIsWow64Process)
             {
                 if (!fnIsWow64Process(GetCurrentProcess(),&bIsWow64))
@@ -50,7 +49,7 @@ int IsWow64(void)
                     // on error, assume FALSE.
                     // in other words, do nothing.
                 }
-            }       
+            }
             FreeLibrary(hModule);
         }
         init = FALSE;
@@ -303,7 +302,7 @@ long
 RegDeleteEntryAlt(const char *entryName, regentry_t entryType)
 {
     long status = ERROR_SUCCESS;
-    char *entryBuf = _strdup(entryName);
+    char *entryBuf = strdup(entryName);
 
     if (entryBuf == NULL) {
 	status = ERROR_NOT_ENOUGH_MEMORY;
