@@ -35,20 +35,19 @@ extern afs_rwlock_t afs_xcbhash;
 
 int
 afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs, 
-     register struct vcache **avcp, afs_ucred_t *acred)
+     struct vcache **avcp, afs_ucred_t *acred)
 {
-    struct vrequest treq;
-    register afs_int32 code;
-    register struct afs_conn *tc;
+    struct vrequest *treq = NULL;
+    afs_int32 code;
+    struct afs_conn *tc;
+    struct rx_connection *rxconn;
     struct VenusFid newFid;
-    register struct dcache *tdc;
-#ifdef AFS_DISCON_ENV
+    struct dcache *tdc;
     struct dcache *new_dc;
-#endif
     afs_size_t offset, len;
-    register struct vcache *tvc;
+    struct vcache *tvc;
     struct AFSStoreStatus InStatus;
-    struct AFSFetchStatus OutFidStatus, OutDirStatus;
+    struct AFSFetchStatus *OutFidStatus, *OutDirStatus;
     struct AFSCallBack CallBack;
     struct AFSVolSync tsync;
     afs_int32 now;
@@ -60,7 +59,11 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     afs_Trace2(afs_iclSetp, CM_TRACE_MKDIR, ICL_TYPE_POINTER, adp,
 	       ICL_TYPE_STRING, aname);
 
-    if ((code = afs_InitReq(&treq, acred)))
+    OutFidStatus = osi_AllocSmallSpace(sizeof(struct AFSFetchStatus));
+    OutDirStatus = osi_AllocSmallSpace(sizeof(struct AFSFetchStatus));
+    memset(&InStatus, 0, sizeof(InStatus));
+
+    if ((code = afs_CreateReq(&treq, acred)))
 	goto done2;
     afs_InitFakeStat(&fakestate);
 
@@ -76,10 +79,10 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     
     AFS_DISCON_LOCK();
 
-    code = afs_EvalFakeStat(&adp, &fakestate, &treq);
+    code = afs_EvalFakeStat(&adp, &fakestate, treq);
     if (code)
 	goto done;
-    code = afs_VerifyVCache(adp, &treq);
+    code = afs_VerifyVCache(adp, treq);
     if (code)
 	goto done;
 
@@ -100,24 +103,24 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     InStatus.ClientModTime = osi_Time();
     InStatus.UnixModeBits = attrs->va_mode & 0xffff;	/* only care about protection bits */
     InStatus.Group = (afs_int32) afs_cr_gid(acred);
-    tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);
+    tdc = afs_GetDCache(adp, (afs_size_t) 0, treq, &offset, &len, 1);
     ObtainWriteLock(&adp->lock, 153);
 
     if (!AFS_IS_DISCON_RW) {
     	do {
-	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
+	    tc = afs_Conn(&adp->f.fid, treq, SHARED_LOCK, &rxconn);
 	    if (tc) {
 	    	XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_MAKEDIR);
 	    	now = osi_Time();
 	    	RX_AFS_GUNLOCK();
 	    	code =
-		    RXAFS_MakeDir(tc->id,
+		    RXAFS_MakeDir(rxconn,
 		    		(struct AFSFid *)&adp->f.fid.Fid,
 				aname,
 				&InStatus,
 				(struct AFSFid *)&newFid.Fid,
-				&OutFidStatus,
-				&OutDirStatus,
+				OutFidStatus,
+				OutDirStatus,
 				&CallBack,
 				&tsync);
 	    	RX_AFS_GLOCK();
@@ -127,16 +130,12 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    } else
 	    	code = -1;
     	} while (afs_Analyze
-		    (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_MAKEDIR,
+		    (tc, rxconn, code, &adp->f.fid, treq, AFS_STATS_FS_RPCIDX_MAKEDIR,
 		     SHARED_LOCK, NULL));
 
     	if (code) {
 	    if (code < 0) {
-	    	ObtainWriteLock(&afs_xcbhash, 490);
-	    	afs_DequeueCallback(adp);
-	    	adp->f.states &= ~CStatd;
-	    	ReleaseWriteLock(&afs_xcbhash);
-	    	osi_dnlc_purgedp(adp);
+		afs_StaleVCache(adp);
 	    }
 	    ReleaseWriteLock(&adp->lock);
 	    if (tdc)
@@ -145,11 +144,10 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
         }
 
     } else {
-#if defined(AFS_DISCON_ENV)
     	/* Disconnected. */
 
 	/* We have the dir entry now, we can use it while disconnected. */
-	if (adp->mvid == NULL) {
+	if (adp->mvid.target_root == NULL) {
 	    /* If not mount point, generate a new fid. */
 	    newFid.Cell = adp->f.fid.Cell;
     	    newFid.Fid.Volume = adp->f.fid.Fid.Volume;
@@ -160,13 +158,12 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	/* Operations with the actual dir's cache entry are further
 	 * down, where the dir entry gets created.
 	 */
-#endif
     }			/* if (!AFS_IS_DISCON_RW) */
 
     /* otherwise, we should see if we can make the change to the dir locally */
     if (tdc)
 	ObtainWriteLock(&tdc->lock, 632);
-    if (AFS_IS_DISCON_RW || afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
+    if (AFS_IS_DISCON_RW || afs_LocalHero(adp, tdc, OutDirStatus, 1)) {
 	/* we can do it locally */
 	ObtainWriteLock(&afs_xdcache, 294);
 	code = afs_dir_Create(tdc, aname, &newFid.Fid);
@@ -185,12 +182,11 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	/* We will have to settle with the local link count. */
 	adp->f.m.LinkCount++;
     else
-	adp->f.m.LinkCount = OutDirStatus.LinkCount;
+	adp->f.m.LinkCount = OutDirStatus->LinkCount;
     newFid.Cell = adp->f.fid.Cell;
     newFid.Fid.Volume = adp->f.fid.Fid.Volume;
     ReleaseWriteLock(&adp->lock);
     if (AFS_IS_DISCON_RW) {
-#if defined(AFS_DISCON_ENV)
     	/* When disconnected, we have to create the full dir here. */
 
 	/* Generate a new vcache and fill it. */
@@ -198,20 +194,20 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	if (tvc) {
 	    *avcp = tvc;
 	} else {
-	    code = ENOENT;
+	    code = EIO;
 	    goto done;
 	}
 
 	ObtainWriteLock(&tvc->lock, 738);
-	afs_GenDisconStatus(adp, tvc, &newFid, attrs, &treq, VDIR);
+	afs_GenDisconStatus(adp, tvc, &newFid, attrs, treq, VDIR);
 	ReleaseWriteLock(&tvc->lock);
 
 	/* And now make an empty dir, containing . and .. : */
 	/* Get a new dcache for it first. */
-	new_dc = afs_GetDCache(tvc, (afs_size_t) 0, &treq, &offset, &len, 1);
+	new_dc = afs_GetDCache(tvc, (afs_size_t) 0, treq, &offset, &len, 1);
 	if (!new_dc) {
-	    printf("afs_mkdir: can't get new dcache for dir.\n");
-	    code = ENOENT;
+	    /* printf("afs_mkdir: can't get new dcache for dir.\n"); */
+	    code = EIO;
 	    goto done;
 	}
 
@@ -220,8 +216,7 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 			       (afs_int32 *) &newFid.Fid,
 			       (afs_int32 *) &adp->f.fid.Fid);
 	ReleaseWriteLock(&afs_xdcache);
-	if (code)
-	    printf("afs_mkdir: afs_dirMakeDir code = %u\n", code);
+	/* if (code) printf("afs_mkdir: afs_dirMakeDir code = %u\n", code); */
 
 	afs_PutDCache(new_dc);
 
@@ -231,23 +226,29 @@ afs_mkdir(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 
 	afs_DisconAddDirty(tvc, VDisconCreate, 1);
 	ReleaseWriteLock(&tvc->lock);
-#endif				/* #ifdef AFS_DISCON_ENV */
     } else {
     	/* now we're done with parent dir, create the real dir's cache entry */
-    	tvc = afs_GetVCache(&newFid, &treq, NULL, NULL);
+	tvc = afs_GetVCache(&newFid, treq);
     	if (tvc) {
 	    code = 0;
 	    *avcp = tvc;
-    	} else
-	    code = ENOENT;
+
+	} else {
+	    /* For some reason, we cannot fetch the vcache for our
+	     * newly-created dir. */
+	    code = EIO;
+	}
     }				/* if (AFS_DISCON_RW) */
 
   done:
     AFS_DISCON_UNLOCK();
   done3:
     afs_PutFakeStat(&fakestate);
-    code = afs_CheckCode(code, &treq, 26);
+    code = afs_CheckCode(code, treq, 26);
+    afs_DestroyReq(treq);
   done2:
+    osi_FreeSmallSpace(OutFidStatus);
+    osi_FreeSmallSpace(OutDirStatus);
     return code;
 }
 
@@ -261,15 +262,16 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, struct vnode *cdirp,
 afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 #endif
 {
-    struct vrequest treq;
-    register struct dcache *tdc;
-    register struct vcache *tvc = NULL;
-    register afs_int32 code;
-    register struct afs_conn *tc;
+    struct vrequest *treq = NULL;
+    struct dcache *tdc;
+    struct vcache *tvc = NULL;
+    afs_int32 code;
+    struct afs_conn *tc;
     afs_size_t offset, len;
     struct AFSFetchStatus OutDirStatus;
     struct AFSVolSync tsync;
     struct afs_fakestat_state fakestate;
+    struct rx_connection *rxconn;
     XSTATS_DECLS;
     OSI_VC_CONVERT(adp);
 
@@ -278,7 +280,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
     afs_Trace2(afs_iclSetp, CM_TRACE_RMDIR, ICL_TYPE_POINTER, adp,
 	       ICL_TYPE_STRING, aname);
 
-    if ((code = afs_InitReq(&treq, acred)))
+    if ((code = afs_CreateReq(&treq, acred)))
 	goto done2;
     afs_InitFakeStat(&fakestate);
 
@@ -289,11 +291,11 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 
     AFS_DISCON_LOCK();
 
-    code = afs_EvalFakeStat(&adp, &fakestate, &treq);
+    code = afs_EvalFakeStat(&adp, &fakestate, treq);
     if (code)
 	goto done;
 
-    code = afs_VerifyVCache(adp, &treq);
+    code = afs_VerifyVCache(adp, treq);
     if (code)
 	goto done;
 
@@ -311,7 +313,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
         goto done;
     }
 
-    tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);	/* test for error below */
+    tdc = afs_GetDCache(adp, (afs_size_t) 0, treq, &offset, &len, 1);	/* test for error below */
     ObtainWriteLock(&adp->lock, 154);
     if (tdc)
 	ObtainSharedLock(&tdc->lock, 633);
@@ -321,16 +323,14 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 	unlinkFid.Fid.Vnode = 0;
 	code = afs_dir_Lookup(tdc, aname, &unlinkFid.Fid);
 	if (code == 0) {
-	    afs_int32 cached = 0;
-
 	    unlinkFid.Cell = adp->f.fid.Cell;
 	    unlinkFid.Fid.Volume = adp->f.fid.Fid.Volume;
 	    if (unlinkFid.Fid.Unique == 0) {
 		tvc =
-		    afs_LookupVCache(&unlinkFid, &treq, &cached, adp, aname);
+		    afs_LookupVCache(&unlinkFid, treq, adp, aname);
 	    } else {
 		ObtainReadLock(&afs_xvcache);
-		tvc = afs_FindVCache(&unlinkFid, 0, 1 /* do xstats */ );
+		tvc = afs_FindVCache(&unlinkFid, 1 /* do xstats */ );
 		ReleaseReadLock(&afs_xvcache);
 	    }
 	}
@@ -339,12 +339,12 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
     if (!AFS_IS_DISCON_RW) {
 	/* Not disconnected, can connect to server. */
     	do {
-	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
+	    tc = afs_Conn(&adp->f.fid, treq, SHARED_LOCK, &rxconn);
 	    if (tc) {
 	    	XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_REMOVEDIR);
 	    	RX_AFS_GUNLOCK();
 	    	code =
-		    RXAFS_RemoveDir(tc->id,
+		    RXAFS_RemoveDir(rxconn,
 		    		(struct AFSFid *)&adp->f.fid.Fid,
 				aname,
 				&OutDirStatus,
@@ -354,7 +354,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 	    } else
 	    	code = -1;
     	} while (afs_Analyze
-	         (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_REMOVEDIR,
+	         (tc, rxconn, code, &adp->f.fid, treq, AFS_STATS_FS_RPCIDX_REMOVEDIR,
 	         SHARED_LOCK, NULL));
 
     	if (code) {
@@ -364,11 +364,7 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 	    }
 
 	    if (code < 0) {
-	    	ObtainWriteLock(&afs_xcbhash, 491);
-	    	afs_DequeueCallback(adp);
-	    	adp->f.states &= ~CStatd;
-	    	ReleaseWriteLock(&afs_xcbhash);
-	    	osi_dnlc_purgedp(adp);
+		afs_StaleVCache(adp);
 	    }
 	    ReleaseWriteLock(&adp->lock);
 	    goto done;
@@ -378,12 +374,11 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
     	adp->f.m.LinkCount = OutDirStatus.LinkCount;
 
     } else {
-#if defined(AFS_DISCON_ENV)
     	/* Disconnected. */
 
 	if (!tdc) {
 	    ReleaseWriteLock(&adp->lock);
-	    printf("afs_rmdir: No local dcache!\n");
+	    /* printf("afs_rmdir: No local dcache!\n"); */
 	    code = ENETDOWN;
 	    goto done;
 	}
@@ -397,11 +392,11 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 	    code = afs_dir_Lookup(tdc, aname, &tfid.Fid);
 
 	    ObtainSharedLock(&afs_xvcache, 764);
-	    tvc = afs_FindVCache(&tfid, 0, 1 /* do xstats */ );
+	    tvc = afs_FindVCache(&tfid, 1 /* do xstats */ );
 	    ReleaseSharedLock(&afs_xvcache);
 	    
 	    if (!tvc) {
-		printf("afs_rmdir: Can't find dir's vcache!\n");
+		/* printf("afs_rmdir: Can't find dir's vcache!\n"); */
 		ReleaseSharedLock(&tdc->lock);
 	        afs_PutDCache(tdc);	/* drop ref count */
     	        ReleaseWriteLock(&adp->lock);
@@ -431,7 +426,6 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 	}
 
 	adp->f.m.LinkCount--;
-#endif				/* #ifdef AFS_DISCON_ENV */
     }				/* if (!AFS_IS_DISCON_RW) */
 
     if (tdc)
@@ -457,7 +451,6 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
     if (tvc) {
 	ObtainWriteLock(&tvc->lock, 155);
 	tvc->f.states &= ~CUnique;	/* For the dfs xlator */
-#if defined(AFS_DISCON_ENV)
 	if (AFS_IS_DISCON_RW) {
 	    if (tvc->f.ddirty_flags & VDisconCreate) {
 		/* If we we were created whilst disconnected, removal doesn't
@@ -467,7 +460,6 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
 		afs_DisconAddDirty(tvc, VDisconRemove, 1);
 	    }
 	}
-#endif
 	ReleaseWriteLock(&tvc->lock);
 	afs_PutVCache(tvc);
     }
@@ -478,7 +470,8 @@ afs_rmdir(OSI_VC_DECL(adp), char *aname, afs_ucred_t *acred)
   done:
     AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
-    code = afs_CheckCode(code, &treq, 27);
+    code = afs_CheckCode(code, treq, 27);
+    afs_DestroyReq(treq);
   done2:
     return code;
 }
