@@ -54,7 +54,7 @@ static int afs_dynrootInit = 0;
 static int afs_dynrootEnable = 0;
 static int afs_dynrootCell = 0;
 
-static afs_rwlock_t afs_dynrootDirLock;
+afs_rwlock_t afs_dynrootDirLock;
 /* Start of variables protected by afs_dynrootDirLock */
 static char *afs_dynrootDir = NULL;
 static int afs_dynrootDirLen;
@@ -74,7 +74,7 @@ struct afs_dynSymlink {
     char *target;
 };
 
-static afs_rwlock_t afs_dynSymlinkLock;
+afs_rwlock_t afs_dynSymlinkLock;
 /* Start of variables protected by afs_dynSymlinkLock */
 static struct afs_dynSymlink *afs_dynSymlinkBase = NULL;
 static int afs_dynSymlinkIndex = 0;
@@ -107,6 +107,17 @@ afs_dynrootCellInit(void)
     return 0;
 }
 
+/*!
+ * Returns non-zero if the volume is the dynroot volume.
+ */
+int
+afs_IsDynrootVolume(struct volume *v)
+{
+    return (afs_dynrootEnable
+	    && v->cell == afs_dynrootCell
+	    && v->volume == AFS_DYNROOT_VOLUME);
+}
+
 /*
  * Returns non-zero iff fid corresponds to the top of the dynroot volume.
  */
@@ -205,18 +216,24 @@ afs_dynroot_computeDirEnt(char *name, int *curPageP, int *curChunkP)
  * Add directory entry by given name to a directory.  Assumes the
  * caller has allocated the directory to be large enough to hold
  * the necessary entry.
+ *
+ * In order to make sure that memory isn't being accidentally stomped, receive
+ * the size of the directory as an argument (dirSize) and use this info to
+ * calculate the maximum number of bytes that can be occupied by the name
+ * mentioned above.
  */
 static void
 afs_dynroot_addDirEnt(struct DirHeader *dirHeader, int *curPageP,
-		      int *curChunkP, char *name, int vnode)
+		      int *curChunkP, char *name, int vnode, size_t dirSize)
 {
     char *dirBase = (char *)dirHeader;
     struct PageHeader *pageHeader;
-    struct DirEntry *dirEntry;
+    struct DirEntryFlex *dirEntry;
     int sizeOfEntry, i, t1, t2;
     int curPage = *curPageP;
     int curChunk = *curChunkP;
     int didNewPage = 0;
+    size_t offset, limit;
 
     /*
      * Check if we need to flip pages..  If so, init the new page.
@@ -240,13 +257,22 @@ afs_dynroot_addDirEnt(struct DirHeader *dirHeader, int *curPageP,
 	dirHeader->alloMap[curPage] = EPP - 1;
     }
 
-    dirEntry = (struct DirEntry *)(pageHeader + curChunk);
+    dirEntry = (struct DirEntryFlex *)(pageHeader + curChunk);
     dirEntry->flag = 1;
     dirEntry->length = 0;
     dirEntry->next = 0;
     dirEntry->fid.vnode = htonl(vnode);
     dirEntry->fid.vunique = htonl(1);
-    strcpy(dirEntry->name, name);
+
+    osi_Assert(dirEntry->name > dirBase);
+    offset = dirEntry->name - dirBase;
+    osi_Assert(dirSize > offset);
+    limit = dirSize - offset;
+    /*
+     * The caller must have ensured that the directory is large enough, so
+     * strlcpy() should never truncate the string.
+     */
+    strlcpy(dirEntry->name, name, limit);
 
     for (i = curChunk; i < curChunk + sizeOfEntry; i++) {
 	t1 = i / 8;
@@ -275,7 +301,6 @@ afs_dynroot_addDirEnt(struct DirHeader *dirHeader, int *curPageP,
 void
 afs_DynrootInvalidate(void)
 {
-    afs_int32 retry;
     struct vcache *tvc;
     struct VenusFid tfid;
 
@@ -288,15 +313,11 @@ afs_DynrootInvalidate(void)
     ReleaseWriteLock(&afs_dynrootDirLock);
 
     afs_GetDynrootFid(&tfid);
-    do {
-	retry = 0;
-	ObtainReadLock(&afs_xvcache);
-	tvc = afs_FindVCache(&tfid, &retry, 0);
-	ReleaseReadLock(&afs_xvcache);
-    } while (retry);
+    ObtainReadLock(&afs_xvcache);
+    tvc = afs_FindVCache(&tfid, 0);
+    ReleaseReadLock(&afs_xvcache);
     if (tvc) {
-	tvc->f.states &= ~(CStatd | CUnique);
-	osi_dnlc_purgedp(tvc);
+	afs_StaleVCacheFlags(tvc, AFS_STALEVC_NOCB, CUnique);
 	afs_PutVCache(tvc);
     }
 }
@@ -348,8 +369,8 @@ afs_RebuildDynroot(void)
 	dotLen = strlen(c->cellName) + 2;
 	dotCell = afs_osi_Alloc(dotLen);
 	osi_Assert(dotCell != NULL);
-	strcpy(dotCell, ".");
-	afs_strcat(dotCell, c->cellName);
+	osi_Assert(strlcpy(dotCell, ".", dotLen) < dotLen);
+	osi_Assert(strlcat(dotCell, c->cellName, dotLen) < dotLen);
 
 	afs_dynroot_computeDirEnt(c->cellName, &curPage, &curChunk);
 	afs_dynroot_computeDirEnt(dotCell, &curPage, &curChunk);
@@ -367,8 +388,8 @@ afs_RebuildDynroot(void)
 	dotLen = strlen(ca->alias) + 2;
 	dotCell = afs_osi_Alloc(dotLen);
 	osi_Assert(dotCell != NULL);
-	strcpy(dotCell, ".");
-	afs_strcat(dotCell, ca->alias);
+	osi_Assert(strlcpy(dotCell, ".", dotLen) < dotLen);
+	osi_Assert(strlcat(dotCell, ca->alias, dotLen) < dotLen);
 
 	afs_dynroot_computeDirEnt(ca->alias, &curPage, &curChunk);
 	afs_dynroot_computeDirEnt(dotCell, &curPage, &curChunk);
@@ -410,10 +431,11 @@ afs_RebuildDynroot(void)
     memset(dirHeader->hashTable, 0, NHASHENT * sizeof(dirHeader->hashTable[0]));
 
     /* Install ".", "..", and the dynamic mount directory */
-    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ".", 1);
-    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, "..", 1);
+    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ".", 1, dirSize);
+    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, "..", 1, dirSize);
     afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk,
-			  AFS_DYNROOT_MOUNTNAME, AFS_DYNROOT_MOUNT_VNODE);
+			  AFS_DYNROOT_MOUNTNAME, AFS_DYNROOT_MOUNT_VNODE,
+			  dirSize);
     linkCount += 3;
 
     for (cellidx = 0; cellidx < maxcellidx; cellidx++) {
@@ -428,12 +450,12 @@ afs_RebuildDynroot(void)
 	dotLen = strlen(c->cellName) + 2;
 	dotCell = afs_osi_Alloc(dotLen);
 	osi_Assert(dotCell != NULL);
-	strcpy(dotCell, ".");
-	afs_strcat(dotCell, c->cellName);
+	osi_Assert(strlcpy(dotCell, ".", dotLen) < dotLen);
+	osi_Assert(strlcat(dotCell, c->cellName, dotLen) < dotLen);
 	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, c->cellName,
-			      VNUM_FROM_CIDX_RW(cellidx, 0));
+			      VNUM_FROM_CIDX_RW(cellidx, 0), dirSize);
 	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, dotCell,
-			      VNUM_FROM_CIDX_RW(cellidx, 1));
+			      VNUM_FROM_CIDX_RW(cellidx, 1), dirSize);
 	afs_osi_Free(dotCell, dotLen);
 
 	linkCount += 2;
@@ -448,12 +470,12 @@ afs_RebuildDynroot(void)
 	dotLen = strlen(ca->alias) + 2;
 	dotCell = afs_osi_Alloc(dotLen);
 	osi_Assert(dotCell != NULL);
-	strcpy(dotCell, ".");
-	afs_strcat(dotCell, ca->alias);
+	osi_Assert(strlcpy(dotCell, ".", dotLen) < dotLen);
+	osi_Assert(strlcat(dotCell, ca->alias, dotLen) < dotLen);
 	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ca->alias,
-			      VNUM_FROM_CAIDX_RW(aliasidx, 0));
+			      VNUM_FROM_CAIDX_RW(aliasidx, 0), dirSize);
 	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, dotCell,
-			      VNUM_FROM_CAIDX_RW(aliasidx, 1));
+			      VNUM_FROM_CAIDX_RW(aliasidx, 1), dirSize);
 	afs_osi_Free(dotCell, dotLen);
 	afs_PutCellAlias(ca);
     }
@@ -461,7 +483,8 @@ afs_RebuildDynroot(void)
     ts = afs_dynSymlinkBase;
     while (ts) {
 	int vnum = VNUM_FROM_TYPEID(VN_TYPE_SYMLINK, ts->index);
-	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ts->name, vnum);
+	afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ts->name, vnum,
+			      dirSize);
 	ts = ts->next;
     }
 
@@ -484,8 +507,9 @@ afs_RebuildDynrootMount(void)
     int curChunk, curPage;
     char *newDir;
     struct DirHeader *dirHeader;
+    size_t dirSize = AFS_PAGESIZE;
 
-    newDir = afs_osi_Alloc(AFS_PAGESIZE);
+    newDir = afs_osi_Alloc(dirSize);
     osi_Assert(newDir != NULL);
 
     /*
@@ -509,14 +533,14 @@ afs_RebuildDynrootMount(void)
     memset(dirHeader->hashTable, 0, NHASHENT * sizeof(dirHeader->hashTable[0]));
 
     /* Install "." and ".." */
-    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ".", 1);
-    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, "..", 1);
+    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, ".", 1, dirSize);
+    afs_dynroot_addDirEnt(dirHeader, &curPage, &curChunk, "..", 1, dirSize);
 
     ObtainWriteLock(&afs_dynrootDirLock, 549);
     if (afs_dynrootMountDir)
 	afs_osi_Free(afs_dynrootMountDir, afs_dynrootMountDirLen);
     afs_dynrootMountDir = newDir;
-    afs_dynrootMountDirLen = AFS_PAGESIZE;
+    afs_dynrootMountDirLen = dirSize;
     ReleaseWriteLock(&afs_dynrootDirLock);
 }
 
@@ -548,7 +572,7 @@ afs_GetDynroot(char **dynrootDir, int *dynrootLen,
 	status->DataVersion = afs_dynrootVersion;
 	status->CallerAccess = PRSFS_LOOKUP | PRSFS_READ;
 	status->AnonymousAccess = PRSFS_LOOKUP | PRSFS_READ;
-	status->UnixModeBits = 0755;
+	status->UnixModeBits = 0555;
 	status->ParentVnode = 1;
 	status->ParentUnique = 1;
 	status->dataVersionHigh = afs_dynrootVersionHigh;
@@ -655,7 +679,8 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 		linklen = strlen(ts->target);
 		avc->linkData = afs_osi_Alloc(linklen + 1);
 		osi_Assert(avc->linkData != NULL);
-		strcpy(avc->linkData, ts->target);
+		osi_Assert(strlcpy(avc->linkData, ts->target, linklen + 1) <
+				   linklen + 1);
 
 		status->Length = linklen;
 		status->UnixModeBits = 0755;
@@ -701,11 +726,12 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 		linklen = 7;
 	    } else {
 		int namelen = strlen(realName);
+		char *prefix = rw ? "." : "";
 		linklen = rw + namelen;
 		avc->linkData = afs_osi_Alloc(linklen + 1);
 		osi_Assert(avc->linkData != NULL);
-		strcpy(avc->linkData, rw ? "." : "");
-		afs_strcat(avc->linkData, realName);
+		osi_Assert(strlcpy(avc->linkData, prefix, linklen + 1) < linklen + 1);
+		osi_Assert(strlcat(avc->linkData, realName, linklen + 1) < linklen + 1);
 	    }
 
 	    status->UnixModeBits = 0755;
@@ -727,16 +753,18 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 	    linklen = 2 + namelen + strlen(bp);
 	    avc->linkData = afs_osi_Alloc(linklen + 1);
 	    osi_Assert(avc->linkData != NULL);
-	    strcpy(avc->linkData, "%");
-	    afs_strcat(avc->linkData, c->cellName);
-	    afs_strcat(avc->linkData, ":");
-	    afs_strcat(avc->linkData, bp);
+	    osi_Assert(strlcpy(avc->linkData, "%", linklen + 1) < linklen + 1);
+	    osi_Assert(strlcat(avc->linkData, c->cellName, linklen + 1) < linklen + 1);
+	    osi_Assert(strlcat(avc->linkData, ":", linklen + 1) < linklen + 1);
+	    osi_Assert(strlcat(avc->linkData, bp, linklen + 1) < linklen + 1);
 
 	    status->UnixModeBits = 0644;
 	    status->ParentVnode = AFS_DYNROOT_MOUNT_VNODE;
 	    afs_PutCell(c, READ_LOCK);
 
 	} else {
+	    char *prefix = rw ? "%" : "#";
+
 	    c = afs_GetCellByIndex(cellidx, READ_LOCK);
 	    if (!c) {
 		afs_warn("dynroot vnode inconsistency, can't find cell %d\n",
@@ -751,9 +779,9 @@ afs_DynrootNewVnode(struct vcache *avc, struct AFSFetchStatus *status)
 	    linklen = 1 + namelen + 10;
 	    avc->linkData = afs_osi_Alloc(linklen + 1);
 	    osi_Assert(avc->linkData != NULL);
-	    strcpy(avc->linkData, rw ? "%" : "#");
-	    afs_strcat(avc->linkData, c->cellName);
-	    afs_strcat(avc->linkData, ":root.cell");
+	    osi_Assert(strlcpy(avc->linkData, prefix, linklen + 1) < linklen + 1);
+	    osi_Assert(strlcat(avc->linkData, c->cellName, linklen + 1) < linklen + 1);
+	    osi_Assert(strlcat(avc->linkData, ":root.cell", linklen + 1) < linklen + 1);
 
 	    status->UnixModeBits = 0644;
 	    afs_PutCell(c, READ_LOCK);
@@ -872,6 +900,7 @@ afs_DynrootVOPSymlink(struct vcache *avc, afs_ucred_t *acred,
 		      char *aname, char *atargetName)
 {
     struct afs_dynSymlink *tps;
+    size_t len;
 
     if (afs_cr_uid(acred))
 	return EPERM;
@@ -894,12 +923,14 @@ afs_DynrootVOPSymlink(struct vcache *avc, afs_ucred_t *acred,
     osi_Assert(tps != NULL);
     tps->index = afs_dynSymlinkIndex++;
     tps->next = afs_dynSymlinkBase;
-    tps->name = afs_osi_Alloc(strlen(aname) + 1);
+    len = strlen(aname) + 1;
+    tps->name = afs_osi_Alloc(len);
     osi_Assert(tps->name != NULL);
-    strcpy(tps->name, aname);
-    tps->target = afs_osi_Alloc(strlen(atargetName) + 1);
+    osi_Assert(strlcpy(tps->name, aname, len) < len);
+    len = strlen(atargetName) + 1;
+    tps->target = afs_osi_Alloc(len);
     osi_Assert(tps->target != NULL);
-    strcpy(tps->target, atargetName);
+    osi_Assert(strlcpy(tps->target, atargetName, len) < len);
     afs_dynSymlinkBase = tps;
     ReleaseWriteLock(&afs_dynSymlinkLock);
 
