@@ -15,7 +15,8 @@
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
 #ifdef AFS_AIX_ENV
-#include <sys/adspace.h>	/* for vm_att(), vm_det() */
+# include <sys/adspace.h>	/* for vm_att(), vm_det() */
+# include <sys/malloc.h>        /* pinned_heap */
 #endif
 
 /* osi_Init -- do once per kernel installation initialization.
@@ -26,11 +27,8 @@
 
 afs_lock_t afs_ftf;		/* flush text lock */
 
-#ifdef AFS_SGI53_ENV
+#ifdef AFS_SGI_ENV
 lock_t afs_event_lock;
-#endif
-
-#ifdef AFS_SGI64_ENV
 flid_t osi_flid;
 #endif
 
@@ -40,11 +38,8 @@ afs_ucred_t *afs_osi_credp;
 kmutex_t afs_global_lock;
 #endif
 
-#if defined(AFS_SGI_ENV) && !defined(AFS_SGI64_ENV)
-long afs_global_owner;
-#endif
-
-#if defined(AFS_DARWIN_ENV) 
+#if defined(AFS_DARWIN_ENV)
+thread_t afs_global_owner;
 #ifdef AFS_DARWIN80_ENV
 lck_mtx_t  *afs_global_lock;
 #else
@@ -52,18 +47,18 @@ struct lock__bsd__ afs_global_lock;
 #endif
 #endif
 
-#if defined(AFS_XBSD_ENV) && !defined(AFS_FBSD50_ENV)
+#if defined(AFS_XBSD_ENV) && !defined(AFS_FBSD_ENV)
+# if defined(AFS_NBSD50_ENV)
+kmutex_t afs_global_mtx;
+# else
 struct lock afs_global_lock;
 afs_proc_t *afs_global_owner;
-#endif
-#ifdef AFS_FBSD50_ENV
+# endif
+#elif defined(AFS_FBSD_ENV)
 struct mtx afs_global_mtx;
+struct thread *afs_global_owner;
 #endif
 
-#if defined(AFS_DARWIN_ENV)
-thread_t afs_global_owner;
-#endif /* AFS_DARWIN_ENV */
-
 #if defined(AFS_AIX41_ENV)
 simple_lock_data afs_global_lock;
 #endif
@@ -77,6 +72,12 @@ osi_Init(void)
 
     osi_InitGlock();
 
+    /* Initialize a lock for the kernel hcrypto bits. */
+#ifndef UKERNEL
+    init_hckernel_mutex();
+#endif
+
+
     if (!afs_osicred_initialized) {
 #if defined(AFS_DARWIN80_ENV)
         afs_osi_ctxtp_initialized = 0;
@@ -85,23 +86,31 @@ osi_Init(void)
 #endif
 #if defined(AFS_XBSD_ENV)
 	/* Can't just invent one, must use crget() because of mutex */
-	afs_osi_credp = crdup(osi_curcred());
+	afs_osi_credp =
+	  crdup(osi_curcred());
+#elif defined(AFS_SUN5_ENV)
+	afs_osi_credp = kcred;
+#elif defined(AFS_DARWIN_ENV)
+	afs_osi_credp = afs_osi_Alloc(sizeof(*afs_osi_credp));
+	osi_Assert(afs_osi_credp != NULL);
+	memset(afs_osi_credp, 0, sizeof(*afs_osi_credp));
+# if defined(AFS_DARWIN80_ENV)
+	afs_osi_credp->cr_ref = 1; /* kauth_cred_get_ref needs 1 existing ref */
+# endif
 #else
 	memset(&afs_osi_cred, 0, sizeof(afs_ucred_t));
-#if defined(AFS_LINUX26_ENV)
+#if defined(AFS_LINUX_ENV)
         afs_set_cr_group_info(&afs_osi_cred, groups_alloc(0));
 #endif
-#if defined(AFS_DARWIN80_ENV)
-        afs_osi_cred.cr_ref = 1; /* kauth_cred_get_ref needs 1 existing ref */
-#else
-	crhold(&afs_osi_cred);	/* don't let it evaporate */
+#if !(defined(AFS_LINUX_ENV) && defined(STRUCT_TASK_STRUCT_HAS_CRED))
+	crhold(&afs_osi_cred);  /* don't let it evaporate */
 #endif
 
 	afs_osi_credp = &afs_osi_cred;
 #endif
 	afs_osicred_initialized = 1;
     }
-#ifdef AFS_SGI64_ENV
+#ifdef AFS_SGI_ENV
     osi_flid.fl_pid = osi_flid.fl_sysid = 0;
 #endif
 
@@ -112,7 +121,7 @@ osi_Init(void)
 void
 afs_osi_MaskSignals(void)
 {
-#ifdef AFS_LINUX22_ENV
+#ifdef AFS_LINUX_ENV
     osi_linux_mask();
 #endif
 }
@@ -121,11 +130,14 @@ afs_osi_MaskSignals(void)
 void
 afs_osi_UnmaskRxkSignals(void)
 {
+#ifdef AFS_LINUX_ENV
+    osi_linux_unmaskrxk();
+#endif
 }
 
 /* Two hacks to try and fix afsdb */
-void 
-afs_osi_MaskUserLoop()
+void
+afs_osi_MaskUserLoop(void)
 {
 #ifdef AFS_DARWIN_ENV
     afs_osi_Invisible();
@@ -135,14 +147,6 @@ afs_osi_MaskUserLoop()
 #endif
 }
 
-void 
-afs_osi_UnmaskUserLoop(void)
-{
-#ifdef AFS_DARWIN_ENV
-    afs_osi_fullSigRestore();
-#endif
-}
-
 /* register rxk listener proc info */
 void
 afs_osi_RxkRegister(void)
@@ -153,7 +157,7 @@ afs_osi_RxkRegister(void)
 void
 afs_osi_Invisible(void)
 {
-#ifdef AFS_LINUX22_ENV
+#ifdef AFS_LINUX_ENV
     afs_osi_MaskSignals();
 #elif defined(AFS_SUN5_ENV)
     curproc->p_flag |= SSYS;
@@ -163,6 +167,8 @@ afs_osi_Invisible(void)
 #elif defined(AFS_DARWIN_ENV)
     /* maybe call init_process instead? */
     current_proc()->p_flag |= P_SYSTEM;
+#elif defined(AFS_NBSD50_ENV)
+    /* XXX in netbsd a system thread is more than invisible */
 #elif defined(AFS_XBSD_ENV)
     curproc->p_flag |= P_SYSTEM;
 #elif defined(AFS_SGI_ENV)
@@ -181,91 +187,13 @@ afs_osi_Visible(void)
 #elif defined(AFS_DARWIN_ENV)
     /* maybe call init_process instead? */
     current_proc()->p_flag &= ~P_SYSTEM;
+#elif defined(AFS_NBSD50_ENV)
+    /* XXX in netbsd a system thread is more than invisible */
 #elif defined(AFS_XBSD_ENV)
     curproc->p_flag &= ~P_SYSTEM;
 #endif
 }
 
-#if !defined(AFS_LINUX20_ENV) && !defined(AFS_XBSD_ENV)
-/* set the real time */
-void
-afs_osi_SetTime(osi_timeval_t * atv)
-{
-#if defined(AFS_AIX32_ENV)
-    struct timestruc_t t;
-
-    t.tv_sec = atv->tv_sec;
-    t.tv_nsec = atv->tv_usec * 1000;
-    ksettimer(&t);		/*  Was -> settimer(TIMEOFDAY, &t); */
-#elif defined(AFS_SUN55_ENV)
-    stime(atv->tv_sec);
-#elif defined(AFS_SUN5_ENV)
-    /*
-     * To get more than second resolution we can use adjtime. The problem
-     * is that the usecs from the server are wrong (by now) so it isn't
-     * worth complicating the following code.
-     */
-    struct stimea {
-	time_t time;
-    } sta;
-
-    sta.time = atv->tv_sec;
-
-    stime(&sta, NULL);
-#elif defined(AFS_SGI_ENV)
-    struct stimea {
-	sysarg_t time;
-    } sta;
-
-    AFS_GUNLOCK();
-    sta.time = atv->tv_sec;
-    stime(&sta);
-    AFS_GLOCK();
-#elif defined(AFS_DARWIN_ENV)
-#ifndef AFS_DARWIN80_ENV
-    AFS_GUNLOCK();
-    setthetime(atv);
-    AFS_GLOCK();
-#endif
-#else
-    /* stolen from kern_time.c */
-#ifndef	AFS_AUX_ENV
-    boottime.tv_sec += atv->tv_sec - time.tv_sec;
-#endif
-#ifdef AFS_HPUX_ENV
-    {
-#if !defined(AFS_HPUX1122_ENV)
-	/* drop the setting of the clock for now. spl7 is not
-	 * known on hpux11.22
-	 */
-	register ulong_t s;
-	struct timeval t;
-	t.tv_sec = atv->tv_sec;
-	t.tv_usec = atv->tv_usec;
-	s = spl7();
-	time = t;
-	(void)splx(s);
-	resettodr(atv);
-#endif
-    }
-#else
-    {
-	register int s;
-	s = splclock();
-	time = *atv;
-	(void)splx(s);
-    }
-    resettodr();
-#endif
-#ifdef	AFS_AUX_ENV
-    logtchg(atv->tv_sec);
-#endif
-#endif /* AFS_DARWIN_ENV */
-    AFS_STATCNT(osi_SetTime);
-}
-#endif /* AFS_LINUX20_ENV */
-
-
 void
 shutdown_osi(void)
 {
@@ -276,6 +204,9 @@ shutdown_osi(void)
        afs_osi_ctxtp = NULL;
        afs_osi_ctxtp_initialized = 0;
     }
+#endif
+#if !defined(AFS_HPUX_ENV) && !defined(UKERNEL) && !defined(AFS_DFBSD_ENV) && !defined(AFS_LINUX_ENV)
+    /* LINUX calls this from afs_cleanup() which hooks into module_exit */
     shutdown_osisleep();
 #endif
     if (afs_cold_shutdown) {
@@ -283,9 +214,38 @@ shutdown_osi(void)
     }
 }
 
-#ifndef AFS_OBSD_ENV
+#if !defined(AFS_HPUX_ENV) && !defined(UKERNEL) && !defined(AFS_DFBSD_ENV) && !defined(AFS_DARWIN_ENV)
+/* DARWIN uses locking, and so must provide its own */
+void
+shutdown_osisleep(void)
+{
+    afs_event_t *tmp;
+    int i;
+
+    for (i=0;i<AFS_EVHASHSIZE;i++) {
+	while ((tmp = afs_evhasht[i]) != NULL) {
+	    afs_evhasht[i] = tmp->next;
+	    if (tmp->refcount > 0) {
+		afs_warn("nonzero refcount in shutdown_osisleep()\n");
+	    } else {
+#if defined(AFS_AIX_ENV)
+		xmfree(tmp, pinned_heap);
+#elif defined(AFS_FBSD_ENV)
+		afs_osi_Free(tmp, sizeof(*tmp));
+#elif defined(AFS_SGI_ENV) || defined(AFS_XBSD_ENV) || defined(AFS_SUN5_ENV)
+		osi_FreeSmallSpace(tmp);
+#elif defined(AFS_LINUX_ENV)
+		kfree(tmp);
+#endif
+	    }
+	}
+    }
+}
+#endif
+
+#if !defined(AFS_OBSD_ENV) && !defined(AFS_NBSD40_ENV)
 int
-afs_osi_suser(void *cr)
+afs_osi_suser(afs_ucred_t *cr)
 {
 #if defined(AFS_SUN510_ENV)
     return (priv_policy(cr, PRIV_SYS_SUSER_COMPAT, B_FALSE, EPERM, NULL) == 0);
