@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,8 +14,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afsweb/nsafs.c,v 1.6 2003/07/15 23:14:35 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include <net/if.h>
@@ -183,7 +181,7 @@ nsafs_decode64(char *buf)
 
 
 /*
- * Interface for pioctls - used for unlogging 
+ * Interface for pioctls - used for unlogging
  */
 #include "afs/venus.h"
 int
@@ -307,8 +305,8 @@ nsafs_set_id_from_ints(int viceid, int group0, int group1)
 
     u.u_viceid = viceid;
     crp = u.u_cred;
-    crp->cr_uid = viceid;
-    crp->cr_ruid = viceid;
+    set_cr_uid(crp, viceid);
+    set_cr_ruid(crp, viceid);
     crp->cr_suid = viceid;
     crp->cr_groups[0] = group0;
     crp->cr_groups[1] = group1;
@@ -333,8 +331,8 @@ nsafs_set_id_from_strings(char *viceid, char *group0, char *group1)
     } else {
 	u.u_viceid = getuid();
 	crp = u.u_cred;
-	crp->cr_uid = getuid();
-	crp->cr_ruid = getuid();
+	set_cr_uid(crp, getuid());
+	set_cr_ruid(crp, getuid());
 	crp->cr_suid = getuid();
 	crp->cr_groups[0] = getgid();
 	crp->cr_ngroups = 1;
@@ -918,13 +916,7 @@ nsafs_basic(pblock * pb, Session * sn, Request * rq)
     code = uafs_klog(user, cell, passwd, &reason);
     memset((void *)&passwd[0], 0, NSAFS_PASSWORD_MAX);
     if (code != 0) {
-#if 0
-	sprintf(txtbuf, "%s@%s: %s\n", user, cell, reason);
-	pblock_nvinsert("status", "Login Failed", rq->vars);
-	return nsafs_error_check(EPERM, txtbuf, pb, sn, rq);
-#else /* 0 */
 	return REQ_PROCEED;
-#endif /* 0 */
     }
     expiration = u.u_expiration;
     usr_assert(expiration != 0);
