@@ -1,6 +1,6 @@
 /* Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -51,12 +51,29 @@ extern int bcdb_UseTape(struct budb_tapeEntry *, afs_int32 *);
 extern int bcdb_FinishTape(struct budb_tapeEntry *);
 extern int bcdb_FindTapeSeq(afs_int32 dumpid, afs_int32 tapeSeq,
 		            struct budb_tapeEntry *teptr);
-extern afs_int32 bcdb_AddVolume(register struct budb_volumeEntry *);
-extern afs_int32 bcdb_AddVolumes(register struct budb_volumeEntry *,
+extern afs_int32 bcdb_AddVolume(struct budb_volumeEntry *);
+extern afs_int32 bcdb_AddVolumes(struct budb_volumeEntry *,
 				 afs_int32 );
 extern afs_int32 udbClientInit(int noAuthFlag, int localauth, char *cellName);
 struct ktc_token;
 extern int vldbClientInit(int noAuthFlag, int localauth, char *cellName,
                           struct ubik_client **cstruct, time_t *expires);
+extern int ubik_Call_SingleServer_BUDB_GetVolumes(struct ubik_client *aclient,
+						  afs_int32 aflags,
+						  afs_int32 majorVersion,
+						  afs_int32 flags,
+						  const char *name,
+						  afs_int32 start,
+						  afs_int32 end,
+						  afs_int32 index,
+						  afs_int32 *nextIndex,
+						  afs_int32 *dbUpdate,
+						  budb_volumeList *volumes);
+extern int ubik_Call_SingleServer_BUDB_DumpDB(struct ubik_client *aclient,
+					      afs_int32 aflags,
+					      int firstcall,
+					      afs_int32 maxLength,
+					      charListT *charListPtr,
+					      afs_int32 *flags);
 #endif
 
