@@ -7,12 +7,9 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/comerr/com_err.c,v 1.5.14.1 2007/04/10 18:39:51 shadow Exp $");
+#include <roken.h>
 
 #include "internal.h"
-#include <stdio.h>
-#include <stdarg.h>
 #include "error_table.h"
 #include "com_err.h"
 
