@@ -1,153 +1,62 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
 /*
- * xdr_rx.c.  XDR using RX. 
+ * xdr_rx.c.  XDR using RX.
  */
 
 #include <afsconfig.h>
-#ifdef	KERNEL
-#include "afs/param.h"
-#else
 #include <afs/param.h>
-#endif
-
 
 #ifdef KERNEL
-#include "afs/sysincludes.h"
-#ifndef UKERNEL
-#include "h/types.h"
-#include "h/uio.h"
-#ifdef	AFS_OSF_ENV
-#include <net/net_globals.h>
-#endif /* AFS_OSF_ENV */
-#ifdef AFS_LINUX20_ENV
-#include "h/socket.h"
-#else
-#include "rpc/types.h"
-#endif
-#ifdef  AFS_OSF_ENV
-#undef kmem_alloc
-#undef kmem_free
-#undef mem_alloc
-#undef mem_free
-#undef register
-#endif /* AFS_OSF_ENV */
-#ifdef AFS_LINUX22_ENV
-#ifndef quad_t
-#define quad_t __quad_t
-#define u_quad_t __u_quad_t
-#endif
-#endif
-#include "rx/xdr.h"
-#include "netinet/in.h"
-#else /* !UKERNEL */
-#include "rpc/types.h"
-#include "rpc/xdr.h"
-#endif /* !UKERNEL */
-#include "rx/rx.h"
-
+# include "afs/sysincludes.h"
+# ifndef UKERNEL
+#  include "h/types.h"
+#  include "h/uio.h"
+#  ifdef AFS_LINUX_ENV
+#   include "h/socket.h"
+#   ifndef quad_t
+#    define quad_t __quad_t
+#    define u_quad_t __u_quad_t
+#   endif
+#  endif
+#  include "netinet/in.h"
+# endif /* !UKERNEL */
 #else /* KERNEL */
-#include <sys/types.h>
-#include <stdio.h>
-#ifndef AFS_NT40_ENV
-#include <netinet/in.h>
-#endif
+# include <roken.h>
+#endif /* KERNEL */
+
 #include "rx.h"
 #include "xdr.h"
-#endif /* KERNEL */
-
-/*
- * This section should really go in the param.*.h files.
- */
-#if defined(KERNEL)
-/*
- * kernel version needs to agree with <rpc/xdr.h>
- * except on Linux which does XDR differently from everyone else
- */
-# if defined(AFS_LINUX20_ENV) && !defined(UKERNEL)
-#  define AFS_XDRS_T void *
-# else
-#  define AFS_XDRS_T XDR *
-# endif
-# if defined(AFS_SUN57_ENV)
-#  define AFS_RPC_INLINE_T rpc_inline_t
-# elif defined(AFS_DUX40_ENV)
-#  define AFS_RPC_INLINE_T int
-# elif defined(AFS_LINUX20_ENV) && !defined(UKERNEL)
-#  define AFS_RPC_INLINE_T afs_int32
-# elif defined(AFS_LINUX20_ENV)
-#  define AFS_RPC_INLINE_T int32_t *
-# else
-#  define AFS_RPC_INLINE_T long
-# endif
-#else /* KERNEL */
-/*
- * user version needs to agree with "xdr.h", i.e. <rx/xdr.h>
- */
-#  define AFS_XDRS_T void *
-#  define AFS_RPC_INLINE_T afs_int32
-#endif /* KERNEL */
 
 /* Static prototypes */
-#if (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV)
-static bool_t xdrrx_getint64(AFS_XDRS_T axdrs, long *lp);
-static bool_t xdrrx_putint64(AFS_XDRS_T axdrs, long *lp);
-#endif /* (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV) */
-
-static bool_t xdrrx_getint32(AFS_XDRS_T axdrs, afs_int32 * lp);
-static bool_t xdrrx_putint32(AFS_XDRS_T axdrs, afs_int32 * lp);
-static bool_t xdrrx_getbytes(AFS_XDRS_T axdrs, caddr_t addr,
+static bool_t xdrrx_getint32(XDR *axdrs, afs_int32 * lp);
+static bool_t xdrrx_putint32(XDR *axdrs, afs_int32 * lp);
+static bool_t xdrrx_getbytes(XDR *axdrs, caddr_t addr,
 			     u_int len);
-static bool_t xdrrx_putbytes(AFS_XDRS_T axdrs, caddr_t addr,
+static bool_t xdrrx_putbytes(XDR *axdrs, caddr_t addr,
 			     u_int len);
-static AFS_RPC_INLINE_T *xdrrx_inline(AFS_XDRS_T axdrs, u_int len);
+static afs_int32 *xdrrx_inline(XDR *axdrs, u_int len);
 
 
 /*
  * Ops vector for stdio type XDR
  */
 static struct xdr_ops xdrrx_ops = {
-#if defined(AFS_NT40_ENV) || (defined(AFS_SGI_ENV) && !defined(__c99))
-    /* Windows does not support labeled assigments */
-    xdrrx_getint32,	/* deserialize an afs_int32 */
-    xdrrx_putint32,	/* serialize an afs_int32 */
-    xdrrx_getbytes,	/* deserialize counted bytes */
-    xdrrx_putbytes,	/* serialize counted bytes */
-    NULL,		/* get offset in the stream: not supported. */
-    NULL,		/* set offset in the stream: not supported. */
-    xdrrx_inline,	/* prime stream for inline macros */
-    NULL		/* destroy stream */
-#else
-#if defined(KERNEL) && ((defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV))
-    .x_getint64 = xdrrx_getint64,
-    .x_putint64 = xdrrx_putint64,
-#endif /* defined(KERNEL) && ((defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV)) */
-#if defined(UKERNEL)
-    .x_getlong = xdrrx_getint32,
-    .x_putlong = xdrrx_putint32,
-#elif !(defined(KERNEL) && defined(AFS_SUN57_ENV))
-    .x_getint32 = xdrrx_getint32,	/* deserialize an afs_int32 */
-    .x_putint32 = xdrrx_putint32,	/* serialize an afs_int32 */
-#endif
-    .x_getbytes = xdrrx_getbytes,	/* deserialize counted bytes */
-    .x_putbytes = xdrrx_putbytes,	/* serialize counted bytes */
-    .x_getpostn = NULL,		/* get offset in the stream: not supported. */
-    .x_setpostn = NULL,		/* set offset in the stream: not supported. */
-    .x_inline = xdrrx_inline,		/* prime stream for inline macros */
-    .x_destroy = NULL,			/* destroy stream */
-#if defined(KERNEL) && defined(AFS_SUN57_ENV)
-    .x_control = NULL,
-    .x_getint32 = xdrrx_getint32,	/* deserialize an afs_int32 */
-    .x_putint32 = xdrrx_putint32,	/* serialize an afs_int32 */
-#endif
-#endif
+    AFS_STRUCT_INIT(.x_getint32, xdrrx_getint32),	/* deserialize an afs_int32 */
+    AFS_STRUCT_INIT(.x_putint32, xdrrx_putint32),	/* serialize an afs_int32 */
+    AFS_STRUCT_INIT(.x_getbytes, xdrrx_getbytes),	/* deserialize counted bytes */
+    AFS_STRUCT_INIT(.x_putbytes, xdrrx_putbytes),	/* serialize counted bytes */
+    AFS_STRUCT_INIT(.x_getpostn, NULL),			/* get offset in the stream: not supported. */
+    AFS_STRUCT_INIT(.x_setpostn, NULL),			/* set offset in the stream: not supported. */
+    AFS_STRUCT_INIT(.x_inline,	 xdrrx_inline),		/* prime stream for inline macros */
+    AFS_STRUCT_INIT(.x_destroy,	 NULL),			/* destroy stream */
 };
 
 /*
@@ -168,35 +77,8 @@ xdrrx_create(XDR * xdrs, struct rx_call *call,
 int rx_pin_failed = 0;
 #endif
 
-#if (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV)
 static bool_t
-xdrrx_getint64(AFS_XDRS_T axdrs, long *lp)
-{
-    XDR * xdrs = (XDR *)axdrs;
-    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
-    afs_int32 i;
-
-    if (rx_Read32(call, &i) == sizeof(i)) {
-	*lp = ntohl(i);
-	return TRUE;
-    }
-    return FALSE;
-}
-
-static bool_t
-xdrrx_putint64(AFS_XDRS_T axdrs, long *lp)
-{
-    XDR * xdrs = (XDR *)axdrs;
-    afs_int32 code, i = htonl(*lp);
-    struct rx_call *call = ((struct rx_call *)(xdrs)->x_private);
-
-    code = (rx_Write32(call, &i) == sizeof(i));
-    return code;
-}
-#endif /* (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG != _MIPS_SZINT)) || defined(AFS_HPUX_64BIT_ENV) */
-
-static bool_t
-xdrrx_getint32(AFS_XDRS_T axdrs, afs_int32 * lp)
+xdrrx_getint32(XDR *axdrs, afs_int32 * lp)
 {
     afs_int32 l;
     XDR * xdrs = (XDR *)axdrs;
@@ -205,7 +87,7 @@ xdrrx_getint32(AFS_XDRS_T axdrs, afs_int32 * lp)
     char *saddr = (char *)&l;
     saddr -= STACK_TO_PIN;
     /*
-     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under 
+     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under
      * splnet (disables interrupts), which is set throughout rx, we can't swap in stack
      * pages if we need so we panic. Since sometimes, under splnet, we'll use more than
      * 2K stack we could try to bring the next few stack pages in here before we call the rx
@@ -235,7 +117,7 @@ xdrrx_getint32(AFS_XDRS_T axdrs, afs_int32 * lp)
 }
 
 static bool_t
-xdrrx_putint32(AFS_XDRS_T axdrs, afs_int32 * lp)
+xdrrx_putint32(XDR *axdrs, afs_int32 * lp)
 {
     afs_int32 code, l = htonl(*lp);
     XDR * xdrs = (XDR *)axdrs;
@@ -244,7 +126,7 @@ xdrrx_putint32(AFS_XDRS_T axdrs, afs_int32 * lp)
     char *saddr = (char *)&code;
     saddr -= STACK_TO_PIN;
     /*
-     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under 
+     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under
      * splnet (disables interrupts), which is set throughout rx, we can't swap in stack
      * pages if we need so we panic. Since sometimes, under splnet, we'll use more than
      * 2K stack we could try to bring the next few stack pages in here before we call the rx
@@ -266,7 +148,7 @@ xdrrx_putint32(AFS_XDRS_T axdrs, afs_int32 * lp)
 }
 
 static bool_t
-xdrrx_getbytes(AFS_XDRS_T axdrs, caddr_t addr, u_int len)
+xdrrx_getbytes(XDR *axdrs, caddr_t addr, u_int len)
 {
     afs_int32 code;
     XDR * xdrs = (XDR *)axdrs;
@@ -275,7 +157,7 @@ xdrrx_getbytes(AFS_XDRS_T axdrs, caddr_t addr, u_int len)
     char *saddr = (char *)&code;
     saddr -= STACK_TO_PIN;
     /*
-     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under 
+     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under
      * splnet (disables interrupts), which is set throughout rx, we can't swap in stack
      * pages if we need so we panic. Since sometimes, under splnet, we'll use more than
      * 2K stack we could try to bring the next few stack pages in here before we call the rx
@@ -298,7 +180,7 @@ xdrrx_getbytes(AFS_XDRS_T axdrs, caddr_t addr, u_int len)
 }
 
 static bool_t
-xdrrx_putbytes(AFS_XDRS_T axdrs, caddr_t addr, u_int len)
+xdrrx_putbytes(XDR *axdrs, caddr_t addr, u_int len)
 {
     afs_int32 code;
     XDR * xdrs = (XDR *)axdrs;
@@ -307,7 +189,7 @@ xdrrx_putbytes(AFS_XDRS_T axdrs, caddr_t addr, u_int len)
     char *saddr = (char *)&code;
     saddr -= STACK_TO_PIN;
     /*
-     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under 
+     * Hack of hacks: Aix3.2 only guarantees that the next 2K of stack in pinned. Under
      * splnet (disables interrupts), which is set throughout rx, we can't swap in stack
      * pages if we need so we panic. Since sometimes, under splnet, we'll use more than
      * 2K stack we could try to bring the next few stack pages in here before we call the rx
@@ -345,8 +227,8 @@ xdrrx_setpos(XDR * xdrs, u_int pos)
 }
 #endif
 
-static AFS_RPC_INLINE_T *
-xdrrx_inline(AFS_XDRS_T axdrs, u_int len)
+static afs_int32 *
+xdrrx_inline(XDR *axdrs, u_int len)
 {
     /* I don't know what this routine is supposed to do, but the stdio module returns null, so we will, too */
     return (0);
