@@ -82,7 +82,6 @@ int function_list_index;
 /* static prototypes */
 static void isdefined(definition * defp);
 static void def_struct(definition * defp);
-static void def_program(definition * defp);
 static void def_enum(definition * defp);
 static void def_const(definition * defp);
 static void def_union(definition * defp);
@@ -177,14 +176,12 @@ get_definition(void)
     case TOK_ENUM:
 	def_enum(defp);
 	break;
-    case TOK_PROGRAM:
-	def_program(defp);
-	break;
     case TOK_CONST:
 	def_const(defp);
 	break;
     case TOK_EOF:
-	return (NULL);
+	free(defp);
+	return NULL;
     case TOK_PACKAGE:
 	def_package(defp);
 	break;
@@ -280,64 +277,6 @@ def_struct(definition * defp)
 	STOREVAL(&complex_defined, defp);
 }
 
-static void
-def_program(definition * defp)
-{
-    token tok;
-    version_list *vlist;
-    version_list **vtailp;
-    proc_list *plist;
-    proc_list **ptailp;
-
-    defp->def_kind = DEF_PROGRAM;
-    scan(TOK_IDENT, &tok);
-    defp->def_name = tok.str;
-    scan(TOK_LBRACE, &tok);
-    vtailp = &defp->def.pr.versions;
-    scan(TOK_VERSION, &tok);
-    do {
-	scan(TOK_IDENT, &tok);
-	vlist = ALLOC(version_list);
-	vlist->vers_name = tok.str;
-	scan(TOK_LBRACE, &tok);
-	ptailp = &vlist->procs;
-	do {
-	    plist = ALLOC(proc_list);
-	    get_type(&plist->res_prefix, &plist->res_type, DEF_PROGRAM);
-	    if (streq(plist->res_type, "opaque")) {
-		error("illegal result type");
-	    }
-	    scan(TOK_IDENT, &tok);
-	    plist->proc_name = tok.str;
-	    scan(TOK_LPAREN, &tok);
-	    get_type(&plist->arg_prefix, &plist->arg_type, DEF_PROGRAM);
-	    if (streq(plist->arg_type, "opaque")) {
-		error("illegal argument type");
-	    }
-	    scan(TOK_RPAREN, &tok);
-	    scan(TOK_EQUAL, &tok);
-	    scan_num(&tok);
-	    scan(TOK_SEMICOLON, &tok);
-	    plist->proc_num = tok.str;
-	    *ptailp = plist;
-	    ptailp = &plist->next;
-	    peek(&tok);
-	} while (tok.kind != TOK_RBRACE);
-	*vtailp = vlist;
-	vtailp = &vlist->next;
-	scan(TOK_RBRACE, &tok);
-	scan(TOK_EQUAL, &tok);
-	scan_num(&tok);
-	vlist->vers_num = tok.str;
-	scan(TOK_SEMICOLON, &tok);
-	scan2(TOK_VERSION, TOK_RBRACE, &tok);
-    } while (tok.kind == TOK_VERSION);
-    scan(TOK_EQUAL, &tok);
-    scan_num(&tok);
-    defp->def.pr.prog_num = tok.str;
-    *vtailp = NULL;
-}
-
 static void
 def_enum(definition * defp)
 {
@@ -430,6 +369,8 @@ def_typedef(definition * defp)
 {
     declaration dec;
 
+    memset(&dec, 0, sizeof(dec));
+
     defp->def_kind = DEF_TYPEDEF;
     get_declaration(&dec, DEF_TYPEDEF);
     defp->def_name = dec.name;
@@ -470,7 +411,10 @@ get_declaration(declaration * dec, defkind dkind)
 	}
 	dec->rel = REL_ARRAY;
 	if (peekscan(TOK_RANGLE, &tok)) {
-	    dec->array_max = "~0";	/* unspecified size, use max */
+	    if ((dkind == DEF_INPARAM) || (dkind == DEF_INOUTPARAM)) {
+		error("input arrays must specify a max size");
+	    }
+	    dec->array_max = "~0u";	/* unspecified size, use max */
 	} else {
 	    scan_num(&tok);
 	    dec->array_max = tok.str;
@@ -519,8 +463,8 @@ get_type(char **prefixp, char **typep, defkind dkind)
 	(void)peekscan(TOK_INT, &tok);
 	break;
     case TOK_VOID:
-	if (dkind != DEF_UNION && dkind != DEF_PROGRAM) {
-	    error("voids allowed only inside union and program definitions");
+	if (dkind != DEF_UNION) {
+	    error("voids allowed only inside union definitions");
 	}
 	*typep = tok.str;
 	break;
@@ -581,8 +525,11 @@ def_package(definition * defp)
     scan(TOK_IDENT, &tok);
     defp->def_name = tok.str;
     no_of_stat_funcs = 0;
-    if (PackageIndex++ >= MAX_PACKAGES)
+
+    PackageIndex++;
+    if (PackageIndex >= MAX_PACKAGES)
 	error("Exceeded upper limit of package statements\n");
+
     function_list_index = 0;
     PackagePrefix[PackageIndex] = tok.str;
     if (MasterPrefix == NULL)
@@ -914,8 +861,8 @@ analyze_ProcParams(definition * defp, token * tokp)
 	Proc_listp = &Proc_list->next;
 	decls = ALLOC(decl_list);
 	memset(decls, 0, sizeof(decl_list));
-    if (tokp->kind != TOK_RPAREN)
-        decls->decl = dec;
+	if (tokp->kind != TOK_RPAREN)
+	    decls->decl = dec;
 	*tailp = decls;
 	tailp = &decls->next;
     } while (tokp->kind != TOK_RPAREN);
@@ -1009,7 +956,7 @@ hdle_param_tok(definition * defp, declaration * dec, token * tokp,
     Proc_list->component_kind = DEF_PARAM;
     Proc_list->code = alloc(250);
     Proc_list->scode = alloc(250);
-    get_declaration(dec, DEF_PARAM);
+    get_declaration(dec, par_kind);
     Proc_list->pl.param_name = dec->name;
     get1_param_type(defp, dec, &Proc_list->pl.param_type);
     print_param(dec);
@@ -1354,7 +1301,12 @@ cs_ProcTail_setup(definition * defp, int split_flag)
 static void
 ss_Proc_CodeGeneration(definition * defp)
 {
-    defp->can_fail = 0;
+    extern char zflag;
+
+    if (zflag)
+	defp->can_fail = 0;
+    else
+	defp->can_fail = 1;
     ss_ProcName_setup(defp);
     if (!cflag) {
 	ss_ProcParams_setup(defp);
@@ -1411,7 +1363,8 @@ ss_ProcParams_setup(definition * defp)
 		}
 		f_print(fout, "\t%s %s", plist->pl.param_type,
 			plist->pl.param_name);
-		*pntr = pres;
+		if (pntr)
+		    *pntr = pres;
 	    } else if (strchr(plist->pl.param_type, '*') == 0) {
 		f_print(fout, "\t%s %s", plist->pl.param_type,
 			plist->pl.param_name);
@@ -1482,8 +1435,6 @@ ss_ProcSpecial_setup(definition * defp)
 		if (streq(string, structname(plist->pl.param_type))) {
 		    plist->pl.string_name = spec->sdef.string_name;
 		    plist->pl.param_flag |= FREETHIS_PARAM;
-		    fprintf(fout, "\n\t%s.%s = 0;", plist->pl.param_name,
-			    spec->sdef.string_name);
 		}
 	    }
 	}
@@ -1498,22 +1449,13 @@ ss_ProcSpecial_setup(definition * defp)
 		    case REL_ARRAY:
 			plist->pl.string_name = alloc(40);
 			if (brief_flag) {
-			    f_print(fout, "\n\t%s.val = 0;",
-				    plist->pl.param_name);
-			    f_print(fout, "\n\t%s.len = 0;",
-				    plist->pl.param_name);
 			    s_print(plist->pl.string_name, "val");
 			} else {
-			    f_print(fout, "\n\t%s.%s_val = 0;",
-				    plist->pl.param_name, defp1->def_name);
-			    f_print(fout, "\n\t%s.%s_len = 0;",
-				    plist->pl.param_name, defp1->def_name);
 			    s_print(plist->pl.string_name, "%s_val",
 				    defp1->def_name);
 			}
 			break;
 		    case REL_POINTER:
-			f_print(fout, "\n\t%s = 0;", plist->pl.param_name);
 			plist->pl.string_name = NULL;
 			break;
 		    default:
@@ -1529,13 +1471,19 @@ ss_ProcSpecial_setup(definition * defp)
 	    if (plist->component_kind == DEF_PARAM) {
 		if (streq(defp1->def_name, structname(plist->pl.param_type))) {
 		    plist->pl.param_flag |= FREETHIS_PARAM;
-		    fprintf(fout, "\n\tmemset(&%s, 0, sizeof(%s));",
-				 plist->pl.param_name, defp1->def_name);
 		}
 	    }
 	}
     }
 
+    for (plist = defp->pc.plists; plist; plist = plist->next) {
+	if (plist->component_kind == DEF_PARAM) {
+	    fprintf(fout, "\n\tmemset(&%s, 0, sizeof(%s));",
+		    plist->pl.param_name,
+		    plist->pl.param_name);
+	}
+    }
+
     f_print(fout, "\n");
 }
 
@@ -1598,6 +1546,8 @@ ss_ProcCallRealProc_setup(definition * defp)
     f_print(fout, ");\n");
     if (zflag) {
 	f_print(fout, "\tif (z_result)\n\t\treturn z_result;\n");
+    } else {
+	f_print(fout, "\tif (z_result)\n\t\tgoto fail;\n");
     }
 }
 
@@ -1818,9 +1768,6 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
 
     f_print(fout, "{\tafs_int32 rcode, code, newHost, thisHost, i, _ucount;\n");
     f_print(fout, "\tint chaseCount, pass, needsync;\n");
-#if 0 /* goes with block below */
-    f_print(fout, "\tint j, inlist;\n");
-#endif
     f_print(fout, "\tstruct rx_connection *tc;\n");
     f_print(fout, "\tstruct rx_peer *rxp;\n");
     f_print(fout, "\tshort origLevel;\n\n");
@@ -1831,17 +1778,6 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
     f_print(fout, "\torigLevel = aclient->initializationState;\n");
     f_print(fout, "\trcode = UNOSERVERS;\n");
     f_print(fout, "\tchaseCount = needsync = 0;\n\n");
-#if 0 /* We should do some sort of caching algorithm for this, but I need to think about it - shadow 26 jun 06 */
-    f_print(fout, "\tinlist = 0;\n");
-    f_print(fout, "\tLOCK_UCLNT_CACHE;\n");
-    f_print(fout, "\tfor (j = 0; ((j < SYNCCOUNT) && calls_needsync[j]); j++) {\n");
-    f_print(fout, "\t\tif (calls_needsync[j] == (int *)%s%s%s) {\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
-    f_print(fout, "\t\t\tinlist = needsync = 1;\n");
-    f_print(fout, "\t\t\tbreak;\n");
-    f_print(fout, "\t\t}\n");
-    f_print(fout, "\t}\n");
-    f_print(fout, "\tUNLOCK_UCLNT_CACHE;\n");
-#endif
     f_print(fout, "\t/* \n\t* First  pass, we try all servers that are up.\n\t* Second pass, we try all servers.\n\t*/\n");
     f_print(fout, "\tfor (pass = 0; pass < 2; pass++) {  /*p */\n");
     f_print(fout, "\t\t/* For each entry in our servers list */\n");
@@ -1928,15 +1864,6 @@ ucs_ProcTail_setup(definition * defp, int split_flag)
     f_print(fout, "\tdone:\n");
     f_print(fout, "\tif (needsync) {\n");
 
-#if 0 /* We should do some sort of caching algorithm for this, but I need to think about it - shadow 26 jun 06 */
-    f_print(fout, "\t\tif (!inlist) {          /* Remember proc call that needs sync site */\n");
-    f_print(fout, "\t\t\tLOCK_UCLNT_CACHE;\n");
-    f_print(fout, "\t\t\tcalls_needsync[synccount % SYNCCOUNT] = (int *)%s%s%s;\n", prefix, PackagePrefix[PackageIndex], defp->pc.proc_name);
-    f_print(fout, "\t\t\tsynccount++;\n");
-    f_print(fout, "\t\t\tUNLOCK_UCLNT_CACHE;\n");
-    f_print(fout, "\t\t\tinlist = 1;\n");
-    f_print(fout, "\t\t}\n");
-#endif
     f_print(fout, "\t\tif (!rcode) {           /* Remember the sync site - cmd successful */\n");
     f_print(fout, "\t\t\trxp = rx_PeerOf(aclient->conns[_ucount]);\n");
     f_print(fout, "\t\t\taclient->syncSite = rx_HostOf(rxp);\n");
@@ -2030,7 +1957,6 @@ er_ProcProcsArray_setup(void)
 			"\nstatic afs_int32 (*StubProcsArray%d[])(struct rx_call *z_call, XDR *z_xdrs) = {_%s%s%s",
 			PackageIndex, prefix, defp->pc.proc_prefix,
 			((definition *) listp->val)->pc.proc_name);
-		defp = (definition *) listp->val;
 	    }
 	}
 	listp = listp->next;
@@ -2222,8 +2148,14 @@ er_TailofOldStyleProc_setup(void)
 static void
 h_ProcMainBody_setup(void)
 {
+    char *pprefix = PackagePrefix[PackageIndex];
     f_print(fout,"\nextern int %s%sExecuteRequest(struct rx_call *);\n",
-	    prefix, PackagePrefix[PackageIndex]);
+	    prefix, pprefix);
+    f_print(fout,"extern char * %s%sTranslateOpCode(int op);\n", prefix,
+	    pprefix);
+    f_print(fout,"extern struct %s%sstats *%s%sOpCodeStats(int op);\n",
+	    prefix, pprefix,
+	    prefix, pprefix);
 }
 
 static void
@@ -2235,6 +2167,10 @@ h_HeadofOldStyleProc_setup(void)
     f_print(fout,"\nextern int %s%sExecuteRequest(struct rx_call *);\n",
 	    prefix, pprefix);
     f_print(fout,"\nextern int %sOpCodeIndex(int op);\n", PackagePrefix[PackageIndex]);
+    f_print(fout, "extern char * %s%sTranslateOpCode(int op);\n",
+	    prefix, pprefix);
+    f_print(fout, "extern struct %s%sstats *%s%sOpCodeStats(int op);\n",
+	    prefix, pprefix, prefix, pprefix);
 }
 
 void
