@@ -50,10 +50,9 @@
 
 # ifdef AFS_NT40_ENV
 #  include <rpc.h>
-#  include <process.h>
 # else
 #  include <net/if.h>
-#  if !defined(AFS_LINUX20_ENV) && !defined(AFS_ARM_DARWIN_ENV)
+#  if !defined(AFS_LINUX_ENV) && !defined(AFS_ARM_DARWIN_ENV)
 #   include <netinet/if_ether.h>
 #  endif
 # endif
@@ -64,6 +63,9 @@
 # include "afs/sysincludes.h"
 # include "afsincludes.h"
 #endif
+#ifdef UKERNEL
+# include "rx/rx_prototypes.h"
+#endif
 
 typedef struct {
     char eaddr[6];		/* 6 bytes of ethernet hardware address */
@@ -96,7 +98,7 @@ void uuid__get_os_time(uuid_time_t * os_time);
  * +--------------------------...-----+
  */
 
-afsUUID afs_uuid_g_nil_uuid = { 0 };
+afsUUID afs_uuid_g_nil_uuid;
 static uuid_time_t time_now, time_last;
 static u_short uuid_time_adjust, clock_seq;
 static afs_uint32 rand_m, rand_ia, rand_ib, rand_irand, uuid_init_done = 0;
@@ -391,7 +393,7 @@ uuid_get_address(uuid_address_p_t addr)
 void
 uuid__get_os_time(uuid_time_t * os_time)
 {
-    osi_timeval_t tp;
+    osi_timeval32_t tp;
 
     osi_GetTime(&tp);
     os_time->hi = tp.tv_sec;
@@ -405,38 +407,37 @@ static int
 uuid_get_address(uuid_address_p_t addr)
 {
     afs_int32 code;
-    afs_uint32 addr1;
-    struct hostent *he;
+    afs_uint32 addr1 = 0;
+    struct hostent *he = NULL;
 
     code = gethostname(hostName1, 64);
-    if (code) {
-	printf("gethostname() failed\n");
-#ifdef AFS_NT40_ENV
-	return ENOENT;
-#else
-	return errno;
-#endif
-    }
-    he = gethostbyname(hostName1);
-    if (!he) {
-	printf("Can't find address for '%s'\n", hostName1);
-#ifdef AFS_NT40_ENV
-	return ENOENT;
-#else
-	return errno;
-#endif
-    } else {
+    if (!code)
+	he = gethostbyname(hostName1);
+
+    if (he)
 	memcpy(&addr1, he->h_addr_list[0], 4);
-	addr1 = ntohl(addr1);
-	memcpy(addr->eaddr, &addr1, 4);
-	addr->eaddr[4] = 0xaa;
-	addr->eaddr[5] = 0x77;
-#ifdef  UUID_DEBUG
-	printf("uuid_get_address: %02x-%02x-%02x-%02x-%02x-%02x\n",
-	       addr->eaddr[0], addr->eaddr[1], addr->eaddr[2], addr->eaddr[3],
-	       addr->eaddr[4], addr->eaddr[5]);
+#ifdef UKERNEL
+    else
+	addr1=rxi_getaddr();
+#endif
+
+    if (!addr1) {
+#ifdef AFS_NT40_ENV
+	return ENOENT;
+#else
+	return errno;
 #endif
     }
+
+    addr1 = ntohl(addr1);
+    memcpy(addr->eaddr, &addr1, 4);
+    addr->eaddr[4] = 0xaa;
+    addr->eaddr[5] = 0x77;
+#ifdef  UUID_DEBUG
+    printf("uuid_get_address: %02x-%02x-%02x-%02x-%02x-%02x\n",
+	   addr->eaddr[0], addr->eaddr[1], addr->eaddr[2], addr->eaddr[3],
+	   addr->eaddr[4], addr->eaddr[5]);
+#endif
     return 0;
 }
 
